@@ -32,37 +32,31 @@ class NaiveBayesClassifier:
 
     """
 
-    def __init__(self, dataset=None):
-        """Naive Bayes Classifier Constructor.
-
-        Trains classifier when dataset is given, otherwise use train method.
-
-        Args:
-            dataset (list, optional): Dataset for training.
-
-        """
-
-        if dataset != None:
-            self.train(dataset)
-
-    def train(self, dataset):
+    def train(self, x, y):
         """Train method.
 
         Trains classifier with dataset.
 
         Args:
-            dataset (pandas.DataFrame or list of tuple): Feature sets + labels
+            x (pandas.DataFrame or list of dict): Feature sets
+            y (pandas.Series or list): Labels
 
         Raises:
-            Exception: If input is not valid.
+            Exception: If input data is not valid.
 
         """
-        if type(dataset) == pd.DataFrame:
-            self._dataset = dataset
-        elif self._is_pairs_format(dataset):
-            self._dataset = self._pairs_to_dataframe(dataset)
+        try:
+            self._y = pd.Series(y)
+        except TypeError:
+            raise Exception('Y must be an array-like object.')
+        if type(x) == pd.DataFrame:
+            self._x = x
+        elif self._is_featuresets_format(x):
+            self._x = self._featuresets_to_dataframe(x)
         else:
             raise Exception('Dataset type is not supported.')
+        self._dataset = self._x.copy()
+        self._dataset['label'] = self._y
 
         self._train_classifier()
 
@@ -92,8 +86,6 @@ class NaiveBayesClassifier:
         else:
             raise Exception('Data type is not supported.')
 
-        self._dataframe_validation()
-
         labels = list()
         for i in range(len(x)):
             labels.append(self._classify_data(x.iloc[i]))
@@ -119,30 +111,30 @@ class NaiveBayesClassifier:
 
     def _train_classifier(self):
 
-        self._n = len(self._dataset)
+        self._n = min(len(self._x), len(self._y))
 
-        self._features = list(self._dataset.columns[:-1])
-        self._labels = list(self._dataset.iloc[:, -1].unique())
+        self._features = list(self._x.columns)
+        self._labels = list(self._y.unique())
 
         self._prior = dict()
         for label in self._labels:
             self._prior[str(label)] = \
-                self._dataset.iloc[:, -1].value_counts()[label]
+                self._y.value_counts()[label]
 
         self._statistics = dict()
         for label in self._labels:
             features = dict()
             for feature in self._features:
-                if self._dataset[feature].dtype in _numeric:
+                if self._x[feature].dtype in _numeric:
                     features[feature] = {
                         'mean': np.mean(
                             self._dataset[
-                                self._dataset.iloc[:, -1] == label
+                                self._dataset['label'] == label
                             ][feature]
                         ),
                         'var': np.var(
                             self._dataset[
-                                self._dataset.iloc[:, -1] == label
+                                self._dataset['label'] == label
                             ][feature]
                         ),
                     }
@@ -183,22 +175,6 @@ class NaiveBayesClassifier:
 
         return np.prod(p)
 
-    def _dataframe_validation(self):
-        pass
-
-    def _is_pairs_format(self, input):
-
-        if not isinstance(input, list):
-            return False
-        elif not all(isinstance(item, tuple) for item in input):
-            return False
-        elif not all(len(item) == 2 for item in input):
-            return False
-        elif not all(isinstance(item[0], dict) for item in input):
-            return False
-
-        return True
-
     def _is_featuresets_format(self, input):
 
         if not isinstance(input, list):
@@ -208,19 +184,6 @@ class NaiveBayesClassifier:
 
         return True
 
-    def _pairs_to_dataframe(self, pairs):
-        data = dict()
-        features = pairs[0][0].keys()
-
-        for feature in features:
-            data[feature] = pd.Series(pair[0][feature] for pair in pairs)
-        labels = pd.Series(pair[1] for pair in pairs)
-
-        df = pd.DataFrame(data=data)
-        df['label'] = labels
-
-        return df
-
     def _featuresets_to_dataframe(self, featuresets):
         data = dict()
         features = featuresets[0].keys()
